/* =====================================================================
 * NetArduino for showing weather forecast and time from the internet
 * ---------------------------------------------------------------------
 * weatherdata.cpp
 *		Handles storage, print and send of weather information
 * =====================================================================
 * By: Zakie Mashiah
 * =====================================================================
 * Copyright
 *  You may use this code with single condition of giving visible
 *  credit to the author.
 * =====================================================================
 * TODO
 * =====================================================================
 *  Revision History
 *      31-Oct-2011 v0.9
 * =====================================================================
 */
#include <avr/pgmspace.h>
#include <WProgram.h>
#include <string.h>

#include "weatherdata.h"
#include "utilfuncs.h"

void WeatherData::Reset() { memset(&data, 0, sizeof(data)); }

void WeatherData::StartForecast() { data.forecastLen = 0; }

	



void WeatherData::Print()
{
	int i;

	Console_printlnP(PSTR("======[  Weather Information ]======"));
	Console_printP(PSTR("City:\t"));				Console.println(data.info.city);
	Console_printlnP(PSTR("Current Conditions"));
	Console_printP(PSTR("\tTemp:\t\t"));			Console.println(data.curr.temp);
	Console_printP(PSTR("\tHumidity:\t"));			Console.println(data.curr.humidity);
	Console_printP(PSTR("\tWind:\t\t"));			Console.println(data.curr.wind);
	Console_printP(PSTR("\tCondition:\t")); 		Console.println(data.curr.condition);
	Console_printP(PSTR("\tIcon:\t\t"));			Console.println(data.curr.icon);
	Console_printlnP(PSTR("Forecast Conditions"));
	for(i=0; i<data.forecastLen; i++)
	{
		Console_printP(PSTR("\tDayOfWeek:\t"));		Console.println(data.forecast[i].dayOfWeek);
		Console_printP(PSTR("\tLow Temp:\t\t"));	Console.println(data.forecast[i].lowTemp);
		Console_printP(PSTR("\tHigh Temp:\t")); 	Console.println(data.forecast[i].highTemp);
		Console_printP(PSTR("\tCondition:\t"));		Console.println(data.forecast[i].condition);
		Console_printP(PSTR("\tIcon:\t\t"));		Console.println(data.forecast[i].icon);
		Console.println();

	}
	Console_printlnP(PSTR("==============[ Done ]============="));
}

void WeatherData::sendElement(char *e, int l)
{
	char *ptr = e;
	int i;

	serialDisplay.print((char)'[');
	for(i=0; i<l; i++, ptr++)
		serialDisplay.print( *ptr );
	serialDisplay.print((char)']');
}

void WeatherData::sendMember(char part, char member, int num)
{
	char buff[10];
	sprintf(buff, "%c%c%d", part, member, num);
	sendElement(buff, strlen(buff));
}

void WeatherData::sendMember(char part, char member, int idx, int num)
{
	char buff[10];
	sprintf(buff, "%c%c%d%d", part, member, idx, num);
	sendElement(buff, strlen(buff));
}

	
void WeatherData::sendMember(char part, char member, char *str)
{
    char buff[30];
    
    sprintf(buff, "%c%c%s", part, member, str);
	sendElement(buff, strlen(buff));
}

void WeatherData::sendMember(char part, char member, int idx, char *str)
{
    char buff[30];
    
    sprintf(buff, "%c%c%d%s", part, member, idx, str);
	sendElement(buff, strlen(buff));
}

void WeatherData::Send()
{
	int i;
	
	sendMember('I', 'C',  data.info.city);
	sendMember('C', 'C', data.curr.condition);
	sendMember('C', 'T', data.curr.temp);
	sendMember('C', 'H', data.curr.humidity);
	sendMember('C', 'I', data.curr.icon);
	sendMember('C', 'W', data.curr.wind);
	for(i=0; i<data.forecastLen; i++)
	{
		sendMember('F', 'D', i, data.forecast[i].dayOfWeek);
		sendMember('F', 'L', i, data.forecast[i].lowTemp);
		sendMember('F', 'H', i, data.forecast[i].highTemp);
		sendMember('F', 'I', i, data.forecast[i].icon);
		sendMember('F', 'C', i, data.forecast[i].condition);
	}
}

void WeatherData::setCity(char *city) { strncpy(data.info.city, city, MAX_CITY_NAME); }
void WeatherData::setTempCurr(int temp) { data.curr.temp = temp; }
void WeatherData::setHumidCurr(int humidity) { data.curr.humidity = humidity; }
void WeatherData::setIconCurr(char *icon) { strncpy(data.curr.icon, icon, MAX_ICON_NAME); }
void WeatherData::setWindCurr(char *wind) { strncpy(data.curr.wind, wind, MAX_WIND_DATA); }
void WeatherData::setCondCurr(char *condition) { strncpy(data.curr.condition, condition, MAX_COND_NAME); }
void WeatherData::setDowFore(char *day) { strncpy(data.forecast[data.forecastLen].dayOfWeek, day, MAX_DOW_NAME); }
void WeatherData::setLowTempFore(int temp) { data.forecast[data.forecastLen].lowTemp = temp; }
void WeatherData::setHighTempFore(int temp) { data.forecast[data.forecastLen].highTemp = temp; }
void WeatherData::setIconFore(char *icon) { strncpy(data.forecast[data.forecastLen].icon, icon, MAX_ICON_NAME); }
void WeatherData::addForecast() { if (data.forecastLen < (MAX_FORECAST_DAYS-1)) data.forecastLen++; }
void WeatherData::setCondFore(char *condition) { strncpy(data.forecast[data.forecastLen].condition, condition, MAX_COND_NAME); }